% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Filter signal magnitudes with 1st order, Butterworth, lowpass filter
%
% Usage: [r_ds] = us_filter_lowpass(p_ds, p_cf)
%
% p_ds ... grid data or grid point data, structure
% p_cf ... cut off frequency (unit: Hz), integer, 0.01 <= (p_cf / sample-rate) <= 0.99, default = 55000 Hz
% r_ds ... return: data structure, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_cond_fltbutter(p_ds, p_cf)
  
  % check arguments
  if (nargin < 2)
    p_cf = 55000;
  end
  if (nargin < 1)
    help us_cond_fltbutter;
    error('Missing function arguments!');
  end
  if (not(us_checkstruct(p_ds, 0)))
    help us_cond_fltbutter;
    error('Wrong argument type (data structure)!');
  end
  
  % copy structure
  r_ds = p_ds;
  
  % update structure
  if (us_checkstruct(p_ds, 1)) % grid point
    cfr = p_cf / r_ds.sr; % cut-off-frequency ratio
    r_ds.yy = hlp_fltbutter(r_ds.yy, cfr);
    % update protocol
    %r_ds = us_update_protocol(r_ds, sprintf('us_cond_fltbutter(\'%s\', %d)', r_ds.obj_type, p_cf), 'Filtered grid point signal with 1st order, Butterworth, lowpass filter');
  elseif (us_checkstruct(p_ds, 2)) % grid
    for i = 1 : r_ds.nx
      for j = 1 : r_ds.ny
        cfr = p_cf / r_ds.arr_ds(i, j).sr; % cut-off-frequency ratio
        r_ds.arr_ds(i, j).yy = hlp_fltbutter(r_ds.arr_ds(i, j).yy, cfr);
        % update protocol
        %r_ds.arr_ds(i, j) = us_update_protocol(r_ds.arr_ds(i, j), sprintf('us_cond_fltbutter(\'%s\', %d)', r_ds.arr_ds(i, j).obj_type, p_cf), 'Filtered grid point signal with 1st order, Butterworth, lowpass filter');
      end
    end
    % update protocol
    %r_ds = us_update_protocol(r_ds, sprintf('us_cond_fltbutter(\'%s\', %d)', r_ds.obj_type, p_cf), 'Filtered grid signals with 1st order, Butterworth, lowpass filter');
  end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper function: filter signal with lowpass Butterworth filter, 1st order
%
% Usage: [r_yy] = hlp_fltbutter(p_yy, p_fr)
%
% p_yy ... array of signal magnitudes, double array
% p_fr ... cut off frequency ratio, double (= cut-off-frequency / sample-rate), 0.1 <= p_fr <= 0.9
function [r_yy] = hlp_fltbutter(p_yy, p_fr)
  
  if (p_fr < 0.01)
    warning('Frequency ratio < 0.01!\n  using frequency ratio = 0.01');
  end
  if (p_fr > 0.99)
    warning('Frequency ratio > 0.99!\n  using frequency ratio = 0.99');
  end
  
  [b, a] = butter(1, p_fr, 'low');
  r_yy = filtfilt(b, a, p_yy);
  
end
