% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Truncate right side of signal magnitudes (cut off signal)
%
% Usage: [r_ds] = us_truncate_right(p_ds, p_ns)
%
% p_ds ... grid data or grid point data, structure
% p_ns ... number of remaining samples, integer
% r_ds ... return: data structure, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_cond_truncright(p_ds, p_ns)
  
  % check arguments
  if (nargin < 2)
    p_ns = 0;
  end
  if (nargin < 1)
    help us_cond_truncright;
    error('Missing function arguments!');
  end
  if (not(us_checkstruct(p_ds, 0)))
    help us_cond_truncright;
    error('Wrong argument type (data structure)!');
  end
  
  % copy structure
  r_ds = p_ds;
  
  % update structure
  if (us_checkstruct(p_ds, 1)) % grid point
    r_ds.nn = p_ns;
    r_ds.yy = hlp_truncright(r_ds.yy, p_ns);
    r_ds.xx_n = hlp_truncright(r_ds.xx_n, p_ns);
    r_ds.xx_t = hlp_truncright(r_ds.xx_t, p_ns);
    r_ds.xx_d = hlp_truncright(r_ds.xx_d, p_ns);
    % update protocol
    %r_ds = us_update_protocol(r_ds, sprintf('us_cond_truncright(\'%s\', %d)', r_ds.obj_type, p_ns), 'Truncated right side of grid point signal magnitudes (cut off signal)');
  elseif (us_checkstruct(p_ds, 2)) % grid
    for i = 1 : r_ds.nx
      for j = 1 : r_ds.ny
        r_ds.arr_ds(i, j).nn = p_ns;
        r_ds.arr_ds(i, j).yy = hlp_truncright(r_ds.arr_ds(i, j).yy, p_ns);
        r_ds.arr_ds(i, j).xx_n = hlp_truncright(r_ds.arr_ds(i, j).xx_n, p_ns);
        r_ds.arr_ds(i, j).xx_t = hlp_truncright(r_ds.arr_ds(i, j).xx_t, p_ns);
        r_ds.arr_ds(i, j).xx_d = hlp_truncright(r_ds.arr_ds(i, j).xx_d, p_ns);
        % update protocol
        %r_ds.arr_ds(i, j) = us_update_protocol(r_ds.arr_ds(i, j), sprintf('us_cond_truncright(\'%s\', %d)', r_ds.arr_ds(i, j).obj_type, p_ns), 'Truncated right side of grid point signal magnitudes (cut off signal)');
      end
    end
    % update protocol
    %r_ds = us_update_protocol(r_ds, sprintf('us_cond_truncright(\'%s\', %d)', r_ds.obj_type, p_ns), 'Truncated right side of grid signal magnitudes (cut off signals)');
  end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper function: truncate right side of signal magnitudes (cut off signal)
%
% Usage: [r_yy] = hlp_truncright(p_yy, p_ns)
%
% p_yy ... array of signal magnitudes, double array
% p_ns ... number of remaining samples, integer, p_ns >= 0, p_ns <= size(p_yy)
function [r_yy] = hlp_truncright(p_yy, p_ns)
  
  if (p_ns >= max(size(p_yy)))
    r_yy = p_yy;
  elseif (p_ns < 1)
    r_yy = [];
  else
    r_yy = p_yy(1 : p_ns);
  end
  
end
