% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Set initial (left-hand-side) signal magnitudes to zero
%
% Usage: [r_ds] = us_cond_zeroleft(p_ds, p_nz)
%
% p_ds ... grid data or grid point data, structure
% p_nz ... set initial signal magnitudes to zero (number of samples), integer
% r_ds ... return: data structure, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_cond_zeroleft(p_ds, p_nz)
  
  % check arguments
  if (nargin < 2)
    p_nz = 0;
  end
  if (nargin < 1)
    help us_cond_zeroleft;
    error('Missing function arguments!');
  end
  if (not(us_checkstruct(p_ds, 0)))
    help us_cond_zeroleft;
    error('Wrong argument type (data structure)!');
  end
  
  % copy structure
  r_ds = p_ds;
  
  % update structure
  if (us_checkstruct(p_ds, 1)) % grid point
    r_ds.yy = hlp_zeroleft(r_ds.yy, p_nz);
    % update protocol
    %r_ds = us_update_protocol(r_ds, sprintf('us_cond_zeroleft(\'%s\', %d)', r_ds.obj_type, p_nz), 'Set left-hand-side of grid point signal magnitudes to zero');
  elseif (us_checkstruct(p_ds, 2)) % grid
    for i = 1 : r_ds.nx
      for j = 1 : r_ds.ny
        r_ds.arr_ds(i, j).yy = hlp_zeroleft(r_ds.arr_ds(i, j).yy, p_nz);
        % update protocol
        %r_ds.arr_ds(i, j) = us_update_protocol(r_ds.arr_ds(i, j), sprintf('us_cond_zeroleft(\'%s\', %d)', r_ds.arr_ds(i, j).obj_type, p_nz), 'Set left-hand-side of grid point signal magnitudes to zero');
      end
    end
    % update protocol
    %r_ds = us_update_protocol(r_ds, sprintf('us_cond_zeroleft(\'%s\', %d)', r_ds.obj_type, p_nz), 'Set left-hand-side of grid signal magnitudes to zero');
  end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper function: set initial signal magnitudes to zero
%
% Usage: [r_yy] = hlp_zeroleft(p_yy, p_nz)
%
% p_yy ... array of signal magnitudes, double array
% p_nz ... set initial signal magnitudes to zero (number of samples), integer, 0 <= p_nz <= size(p_yy)
%
function [r_yy] = hlp_zeroleft(p_yy, p_nz)
  
  if (p_nz >= max(size(p_yy)))
    r_yy = zeros(size(p_yy));
  elseif (p_nz < 1)
    r_yy = p_yy;
  else
    r_yy = p_yy;
    r_yy(1 : p_nz) = 0;
  end
  
end
