% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Create empty data structure
%
% Usage: [r_ds] = us_createstruct(p_tp)
%
% p_tp ... data structure type, integer
%            1: grid point data structure
%            2: grid data structure
%            3: grid point wavelet analysis data structure
%            4: grid wavelet analysis data structure
% r_ds ... return: data structure, structure
%
% p_tp = 1: structure fields, grid point
%  .obj_type ... object type, 'obj_gridpoint'
%  .fp ... data file path (full qualified), string
%  .ix ... grid point index in x direction (horizontal), integer
%  .iy ... grid point index in y direction (vertical), integer
%  .nn ... number of samples, integer
%  .sr ... sample rate (unit: Hz), integer
%  .ss ... speed of sound (unit: m/sec), double
%  .yy ... magnitude series (unit: mVolt), 16-bit integer
%  .xx_n ... series of sample numbers, integer array [1,...,nn]
%  .xx_t ... series of sample times (unit: sec), double array [t0,..,tn]
%  .xx_d ... series of sample distances (unit: m), double array [s0,...,sn]
%  .prot ... analysis protocol, cell array of strings
%
% p_tp = 2: structure fields, grid
%  .obj_type ... object type, 'obj_grid'
%  .dp ... project directory path (full qualified), string
%  .nn ... number of grid points in total, integer
%  .nx ... number of grid points in x direction (horizontal)
%  .ny ... number of grid points in y direction (vertical)
%  .arr_ds ... array of grid point data structures, 2D structure array [ny x nx]
%  .prot ... analysis protocol, cell array of strings
%
% p_tp = 3: structure fields, grid point wavelet analysis
%  .obj_type ... object type, 'obj_wavelet_gridpoint'
%  .fp ... data file path (full qualified), string
%  .ix ... grid point index in x direction (horizontal), integer
%  .iy ... grid point index in y direction (vertical), integer
%  .sr ... sample rate (unit: Hz), integer
%  .ss ... speed of sound (unit: m/sec), double
%  .yy ... signal magnitudes, array of doubles
%  .wt ... wavelet type, string
%  .cc ... cross correlation matrix (normalized), matrix of doubles
%  .wl ... array of wavelet lengths, array of integer
%  .wx ... array of wavelet maxima indeces, array of integer
%
% p_tp = 4: structure fields, grid wavelet analysis
%  .obj_type ... object type, 'obj_wavelet_grid'
%  .dp ... project directory path (full qualified), string
%  .nn ... number of grid points in total, integer
%  .nx ... number of grid points in x direction (horizontal)
%  .ny ... number of grid points in y direction (vertical)
%  .arr_as ... array of grid point wavelet analysis data structures, 2D structure array [ny x nx]
%
function [r_ds] = us_createstruct(p_tp)
  
  % check arguments
  if (nargin < 1)
    help us_createstruct;
    error('Less arguments given!');
  end
  
  % create data structure
  switch (p_tp)
    case 1
      r_ds.obj_type = 'obj_gridpoint';
      r_ds.fp = '';
      r_ds.ix = 0;
      r_ds.iy = 0;
      r_ds.nn = 0;
      r_ds.sr = 0;
      r_ds.ss = 0;
      r_ds.yy = [];
      r_ds.xx_n = [];
      r_ds.xx_t = [];
      r_ds.xx_d = [];
      r_ds.proto = {};
      %r_ds = us_update_protocol(r_ds, 'us_createstruct(1)', '% Grid point analysis protocol');
    case 2
      r_ds.obj_type = 'obj_grid';
      r_ds.dp = '';
      r_ds.nn = 0;
      r_ds.nx = 0;
      r_ds.ny = 0;
      r_ds.arr_ds = struct(us_createstruct(1));
      r_ds.proto = {};
      %r_ds = us_update_protocol(r_ds, 'us_createstruct(2)', '% Grid analysis protocol');
    case 3
      r_ds.obj_type = 'obj_wavelet_gridpoint';
      r_ds.fp = '';
      r_ds.ix = 0;
      r_ds.iy = 0;
      r_ds.sr = 0;
      r_ds.ss = 0;
      r_ds.tp = '';
      r_ds.yy = [];
      r_ds.cc = [];
      r_ds.wl = [];
      r_ds.wx = [];
      r_ds.mv = 0;
      r_ds.mi = 0;
      r_ds.mj = 0;
    case 4
      r_ds.obj_type = 'obj_wavelet_grid';
      r_ds.dp = '';
      r_ds.nn = 0;
      r_ds.nx = 0;
      r_ds.ny = 0;
      r_ds.arr_ds = struct(us_createstruct(3));
    otherwise
      help us_createstruct;
      error('Unknown data structure type!\n  type: %d', p_tp);
  end
  
end
