% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Read one measurement data file (one grid point)
%
% Usage: [r_ds] = us_fread(p_fp, p_sr, p_ss)
%
% p_fp ... grid point data file path (full qualified), string
% p_sr ... sample rate (unit: Hz), integer, default = 5500000
% p_ss ... speed of sound (unit: m/sec), double, default = 1500
% r_ds ... grid point measurement data structure, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_fread(p_fp, p_sr, p_ss)
  
  % check arguments
  if (nargin < 3)
    p_ss = 3000;
  end
  if (nargin < 2)
    p_sr = 25000000;
  end
  if (nargin < 1)
    help us_fread;
    error('Missing function arguments!');
  end
  
  % create empty data structure
  r_ds = us_createstruct(1);
  
  % read data file
  [fid, ferr] = fopen(p_fp);
  if (fid < 0)
    help us_fread;
    error(ferr);
  end
  
  % read data file
  datatype = 'int16';
  % printf('Format: %s\n', datatype);
  [ms, nn] = fread(fid, Inf, datatype);
  
  % update protocol
  %r_ds = us_update_protocol(r_ds, sprintf('us_fread(\'%s\', %d, %.1f)', p_fp, p_sr, p_ss), sprintf('Read data from grid point measurement data file (%d samples)', r_ds.nn));
  
  % delta t for sample times
  dts = 1.0 / p_sr;
  
  % create data structure
  r_ds.fp = p_fp;
  [r_ds.ix, r_ds.iy] = hlp_extract_gpidx(p_fp);
  r_ds.nn = nn;
  r_ds.yy = double(ms);
  r_ds.xx_n = linspace(1, nn, nn);
  r_ds = us_update_smptime(r_ds, p_sr);
  r_ds = us_update_smpdist(r_ds, p_ss);
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper function: Extract grid point indeces from file name
%
% Usage: [r_ix, r_iy] = hlp_extract_gpidx(p_fp)
%
% p_fp ... data file path (full qualified), string
% r_ix ... grid point index in x direction (horizontal), integer
% r_iy ... grid point index in y direction (vertical), integer
%
function [r_ix, r_iy] = hlp_extract_gpidx(p_fp)
  
  % check arguments
  if (isempty(p_fp))
    error('Measurement data file path is empty!');
  end
  
  % extract file name
  [fd, fn, fe] = fileparts(p_fp);
  
  % split file name
  fns = strsplit(fn, '_');
  
  % check number of parts
  if (size(fns, 2) ~= 2)
    error('Measurement data file name has no valid format!\n  file name = %s\n  expected format: <ix>_<iy>.raw', fn);
  end
  
  % return grid point indeces
  r_ix = str2num(fns{1, 1});
  r_iy = str2num(fns{1, 2});
  
end
