% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Plot data structure (signals and wavelet analysis results)
%
% Usage: [r_fh] = us_plot(p_ds, p_tp)
%
% p_ds ... grid data or grid point data, structure
% p_tp ... plot type, integer, default = 1
%           1: plot x=sample-number y=magnitude
%           2: plot x=sample-time, y=magnitude
%           3: plot x=sample-distance, y=magnitude
% r_fh ... return: figure handles, figure_handle array
%
% Note: see us_createstruct for details on structure fields
%
function [r_fh] = us_plot(p_ds, p_tp)
  
  % check arguments
  if (nargin < 2)
    p_tp = 1;
  end
  if (nargin < 1)
    help us_plot;
    error('Missing function arguments!');
  end
  
  % plot settings
  fig_name = 'ultrasonic';
  fig_pos = [300, 300, 800, 400];
  
  % plot
  r_fh = [];
  if (us_checkstruct(p_ds, 1)) % grid point
    fh = figure('name', fig_name, 'position', fig_pos);
    hlp_plot_ms(p_ds, p_tp);
    r_fh = fh;
  elseif (us_checkstruct(p_ds, 2)) % grid
    for i = 1 : p_ds.nx
      for j = 1 : p_ds.ny
        fh = figure('name', fig_name, 'position', fig_pos);
        hlp_plot_ms(p_ds.arr_ds(i, j), p_tp);
        r_fh = [r_fh, fh];
      end
    end
  elseif (us_checkstruct(p_ds, 3)) % grid point wavelet analysis
    fh = figure('name', fig_name, 'position', fig_pos);
    hlp_plot_wl(p_ds, p_tp);
    r_fh = fh;
  elseif (us_checkstruct(p_ds, 4)) % grid wavelet analysis
    for i = 1 : p_ds.nx
      for j = 1 : p_ds.ny
        fh = figure('name', fig_name, 'position', fig_pos);
        hlp_plot_wl(p_ds.arr_ds(i, j), p_tp);
        r_fh = [r_fh, fh];
      end
    end
  end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper function: plot grid point signal magnitudes
%
% Usage: hlp_plot_ms(p_ds, p_tp)
%
% p_ds ... grid or grid point measurement data structure, structure
% p_tp ... plot type, integer
%
function hlp_plot_ms(p_ds, p_tp)
  
  % plot title and legend
  tit = 'Grid point signal';
  fmt = '-r';
  leg = sprintf('GP(%d,%d)', p_ds.iy, p_ds.ix);
  
  % select x data
  switch (p_tp)
    case 1 % sample number
      x_data = p_ds.xx_n;
      x_lbl = 'samples [%]';
      x_scal = 1;
    case 2 % sample time
      x_data = p_ds.xx_t;
      x_lbl = 'time [msec]';
      x_scal = 1000.0;
    case 3 % sample distance
      x_data = p_ds.xx_d;
      x_lbl = 'distance [mm]';
      x_scal = 1000.0;
    otherwise
      error('Unknown plot type!\n  plot type = %d', p_tp);
  end
  
  % select y data
  y_data = p_ds.yy;
  y_lbl = 'signal magnitude [mVolt]';
  y_scal = 1;
  
  % plot signal
  title(tit)
  xlabel(x_lbl);
  ylabel(y_lbl);
  hold on;
  plot(x_data * x_scal, y_data * y_scal, fmt, 'DisplayName', leg, 'LineWidth', 2);
  legend('location', 'northeast');
  hold off;
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper function: plot grid point wavelet analysis
%
% Usage: hlp_plot_wl(p_ds, p_tp)
%
% p_ds ... grid point wavelet analysis data structure, structure
% p_tp ... plot type, integer
%
function hlp_plot_wl(p_ds, p_tp)
  
  % plot title and legend
  tit = 'Grid point wavelet analysis';
  fmts = '-m';
  fmtm = '--r';
  fmt0 = '-g';
  fmt1 = '--b';
  legs = sprintf('GP(%d,%d)', p_ds.iy, p_ds.ix);
  legm = 'Maximum';
  leg0 = 'Begin';
  leg1 = 'End';
  
  % labels
  x_lbl = 'samples [%]';
  y_lbl = 'wavelet length [%]';
  
  % wavelet start, end, max, best fit
  wl2 = fix(p_ds.wl * 0.5);
  xxw0 = p_ds.wx - wl2;
  yyw0 = [1 : size(p_ds.cc, 1)];
  xxw1 = p_ds.wx + wl2;
  yyw1 = [1 : size(p_ds.cc, 1)];
  xxwm = p_ds.wx;
  yywm = [1 : size(p_ds.cc, 1)];
  xfit = p_ds.mj;
  yfit = p_ds.mi;
  lfit = p_ds.wl(p_ds.mi);
  dfit = p_ds.mj - fix(lfit * 0.5);

  % scale and bias signal magnitudes
  yys = fix(p_ds.yy / max(abs(p_ds.yy)) * 0.25 * size(p_ds.cc, 1)) + fix(size(p_ds.cc, 1) / 2);
  xxs = [1 : max(size(yys))];
  
  % plot signal
  % colormap, colorbar
  colormap('gray');
  colorbar;
  % plot title
  title(tit)
  % axis limits
  xlim([1, size(p_ds.cc, 2)]);
  ylim([1, size(p_ds.cc, 1)]);
  % axislabels, ticklabels
  nxticks = max(size(xticks)) + 2;
  nyticks = max(size(yticks)) + 1;
  ytspace = fix(linspace(p_ds.wl(1), p_ds.wl(end), nyticks));
  ytspace(1) = [];
  xtspace = [-1, 0 1];
  switch (p_tp)
    case 1
      xtspace = fix(linspace(1, size(p_ds.cc, 2), nxticks));
      xtspace(1) = [];
      xtspace(end) = [];
      x_lbl = 'samples [%]';
      v_w0 = xxw0(1);
      v_w1 = xxw0(end);
      v_fit = sprintf('S=%d [], N=%d []', lfit, dfit);
    case 2
      xtspace = fix(linspace(0, size(p_ds.cc, 2) / p_ds.sr * 1e6, nxticks));
      xtspace(1) = [];
      x_lbl = 'time [usec]';
      v_w0 = fix(xxw0(1) / p_ds.sr * 1e6);
      v_w1 = fix(xxw0(end) / p_ds.sr * 1e6);
      v_fit = sprintf('S=%d [], T=%d [usec]', lfit, dfit / p_ds.sr * 1e6);
    case 3
      xtspace = fix(linspace(0, p_ds.ss * (size(p_ds.cc, 2) / p_ds.sr * 1e3), nxticks));
      xtspace(1) = [];
      x_lbl = 'distance [mm]';
      v_w0 = fix(p_ds.ss * (xxw0(1) / p_ds.sr) * 1e3);
      v_w1 = fix(p_ds.ss * (xxw0(end) / p_ds.sr) * 1e3);
      v_fit = sprintf('S=%d [], D=%d [mm]', lfit, fix(p_ds.ss * (dfit / p_ds.sr) * 1e3));
  end
  xticklabels(xtspace);
  yticklabels(ytspace);
  xlabel(x_lbl);
  ylabel(y_lbl);
  grid('on');
  hold('on');
  imagesc(p_ds.cc);
  plot(xxw0, yyw0, fmt0, 'DisplayName', leg0, 'LineWidth', 2);
  plot(xxwm, yywm, fmtm, 'DisplayName', legm, 'LineWidth', 1);
  plot(xxw1, yyw1, fmt1, 'DisplayName', leg1, 'LineWidth', 1);
  plot(xxs, yys, fmts, 'DisplayName', legs, 'LineWidth', 1);
  plot(xfit, yfit, '-ro', 'DisplayName', 'best fit', 'LineWidth', 2);
  text(xxw0(1), yyw0(1), sprintf('%d', v_w0), 'color', 'r', 'fontsize', 12);
  text(xxw0(end), yyw0(end), sprintf('%d', v_w1), 'color', 'r', 'fontsize', 12);
  text(xfit, yfit, v_fit, 'color', 'y', 'fontsize', 12);
  hold('off');
  legend('location', 'southwest');
  
end
