% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Plot wavelet function
%
% Usage: [r_fh] = us_plot_wavelet(p_wt, p_wn)
%
% p_wt ... wavelet type, string
%            'mexihat': mexican hat wavelet
%            'cosine1': 1 cosine waves
%            'cosine3': 3 cosine waves
%            'cosine5': 5 cosine waves
% p_wn ... wavelet length (number of samples), integer, p_wn >= 100
% r_fh ... return: figure handle
%
% Note: 
%
function [r_fh] = us_plot_wavelet(p_wt, p_wn)
  
  % check arguments
  if (nargin < 2)
    p_wn = 1000;
  end
  if (nargin < 1)
    help us_plot_wavelet;
    error('Missing function arguments!');
  end
  
  % plot settings
  fig_name = 'ultrasonic';
  fig_pos = [300, 300, 800, 400];
  tit = sprintf('Wavelet function: %s', p_wt);
  fmtw = '-b';
  legw = 'y := w(x)';
  x_lbl = 'x';
  y_lbl = 'y';
  
  % generate wavelet function
  [xx, yy] = us_wavelet_generate(p_wt, p_wn);
  
  % generate limit lines
  xxl0 = [xx(1), xx(1)];
  yyl0 = [-1, 1] * 0.5;
  xxl1 = [xx(end), xx(end)];
  yyl1 = [-1, 1] * 0.5;
  
  % plot wavelet function
  fh = figure('name', fig_name, 'position', fig_pos);
  title(tit);
  grid('on');
  xlabel(x_lbl);
  ylabel(y_lbl);
  hold('on');
  plot(xx, yy, fmtw, 'DisplayName', legw, 'LineWidth', 2);
  plot(xxl0, yyl0, '-r', 'DisplayName', 'begin', 'LineWidth', 2);
  plot(xxl1, yyl1, '-r', 'DisplayName', 'end', 'LineWidth', 2);
  text(xxl0(2), yyl0(2) * 1.1, sprintf('%.2f', xxl0(2)));
  text(xxl1(2), yyl1(2) * 1.1, sprintf('%.2f', xxl1(2)));
  hold('off');
  legend('location', 'northeast');
  
end
