% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Update sample rate and speed of sound
%
% Usage: [r_ds] = us_update_param(p_ds, p_sr, p_ss) ... update sample rate and speed of sound
%        [r_ds] = us_update_param(p_ds, p_sr, []) ... update sample rate only
%        [r_ds] = us_update_param(p_ds, [], p_ss) ... update speed of sound only
%
% p_ds ... grid data or grid point data, structure
% p_sr ... sample rate (unit: Hz), integer
% p_ss ... speed of sound (unit: m/sec), double
% r_ds ... return: data structure, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_update_param(p_ds, p_sr, p_ss)
  
  % check arguments
  if (nargin < 3)
    help us_update_param;
    error('Missing function arguments!');
  end
  if (not(us_checkstruct(p_ds)))
    help us_update_param;
    error('Wrong argument type (data structure)!');
  end
  if (isempty(p_sr))
    upd_sr = false;
  else
    upd_sr = true;
  end
  if (isempty(p_ss))
    upd_ss = false;
  else
    upd_ss = true;
  end
  
  % copy structure
  r_ds = p_ds;
  
  % update structure
  if (us_checkstruct(p_ds, 1)) % grid point
    if (upd_sr)
      r_ds = us_update_smptime(p_ds, p_sr);
    end
    if (upd_ss)
      r_ds = us_update_smpdist(p_ds, p_ss);
    end
  elseif (us_checkstruct(p_ds, 2)) % grid
    for i = 1 : p_ds.nx
      for j = 1 : p_ds.ny
        if (upd_sr)
          r_ds.arr_ds(i, j) = us_update_smptime(p_ds.arr_ds(i, j), p_sr);
        end
        if (upd_ss)
          r_ds.arr_ds(i, j) = us_update_smpdist(p_ds.arr_ds(i, j), p_ss);
        end
        r_ds.arr_ds(i, j) = ds;
      end
    end
  end
  
  % update protocol
  %if (upd_sr)
    %r_ds = us_update_protocol(r_ds, sprintf('us_update_param(\'%s\', %d, [])', p_ds.obj_type, p_sr), 'Updated sample rate for all grid points');
  %end
  %if (upd_ss)
    %r_ds = us_update_protocol(r_ds, sprintf('us_update_param(\'%s\', [], %.1f)', p_ds.obj_type, p_ss), 'Updated speed of sound for all grid points');
  %end
  
end
