% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Run wavelet analysis with given wavelet function and wavelet length range
%
% Usage: [r_ds] = us_wavelet_analyse(p_ds, p_wt, p_wr)
%
% p_ds ... grid data or grid point data, structure
% p_wt ... wavelet type, string (see us_wavelet_generate for details)
% p_wr ... wavelet length range [n_min, n_max], integer array
% r_ds ... return: wavelet analysis data structure, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_wavelet_analyse(p_ds, p_wt, p_wr)
  
  % check arguments
  if (nargin < 3)
    p_wr = [500, 700];
  end
  if (nargin < 2)
    p_wt = 'cosine3';
  end
  if (nargin < 1)
    help us_wavelet_analyse;
    error('Missing function arguments!');
  end
  if (not(us_checkstruct(p_ds, 0)))
    help us_wavelet_analyse;
    error('Wrong argument type (data structure)!');
  end
  if (max(size(p_wr)) ~= 2) % check array size of range
    warning('Sample range is not an array!\n  using sample range = [500, 700]');
    p_wr = [500, 700];
  end
  if (p_wr(1) < 100) % check min size of range
    warning('Lower limit of wavelet size < 100!\n  using lower limit = 100 samples');
    p_wr(1) = 100;
  end
  if (p_wr(2) <= p_wr(1)) % check whether upper limit is greater than lower limit
    warning('Lower limit (%d) is greater than or equal to upper limit (%d)!\n  using upper limit = %d samples', p_wr(1), p_wr(2), p_wr(1) + 10);
    p_wr(2) = p_wr(1) + 10;
  end
  
  % update structure
  if (us_checkstruct(p_ds, 1)) % grid point
    n_max = max(size(p_ds.yy));
    if (p_wr(2) > n_max)
      warning('Upper limit of wavelet size > N / 2 !\n  using upper limit = %d samples', n_max);
      p_wr(2) = n_max;
    end
    r_ds = us_createstruct(3);
    r_ds.fp = p_ds.fp;
    r_ds.ix = p_ds.ix;
    r_ds.iy = p_ds.iy;
    r_ds.sr = p_ds.sr;
    r_ds.yy = p_ds.yy;
    r_ds.wt = p_wt;
    r_ds.wl = [p_wr(1) : p_wr(2)];
    [r_ds.cc, r_ds.wx, r_ds.mv, r_ds.mi, r_ds.mj] = us_wavelet_xcorr(p_ds.yy, p_wt, r_ds.wl);
  elseif (us_checkstruct(p_ds, 2)) % grid
    n_max = max(size(p_ds.arr_ds(1, 1).yy));
    if (p_wr(2) > n_max)
      warning('Upper limit of wavelet size > N / 2 !\n  using upper limit = %d samples', n_max);
      p_wr(2) = n_max;
    end
    r_ds = us_createstruct(4);
    r_ds.dp = p_ds.dp;
    r_ds.nn = p_ds.nn;
    r_ds.nx = p_ds.nx;
    r_ds.ny = p_ds.ny;
    for i = 1 : p_ds.nx
      for j = 1 : p_ds.ny
        ds = us_createstruct(3);
        ds.fp = p_ds.arr_ds(i, j).fp;
        ds.ix = p_ds.arr_ds(i, j).ix;
        ds.iy = p_ds.arr_ds(i, j).iy;
        ds.sr = p_ds.arr_ds(i, j).sr;
        ds.yy = p_ds.arr_ds(i, j).yy;
        ds.wt = p_wt;
        ds.wl = [p_wr(1) : p_wr(2)];
        [ds.cc, ds.wx, ds.mv, ds.mi, ds.mj] = us_wavelet_xcorr(p_ds.arr_ds(i, j).yy, p_wt, ds.wl);
        r_ds.arr_ds(i, j) = ds;
      end
    end
  end
  
end
