% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Generate discrete wavelet function of given type
%
% Usage: [r_xx, r_yy] = us_wavelet_generate(p_wt, p_wn)
%
% p_wt ... wavelet type, string
%            'mexihat': mexican hat wavelet
%            'cos1lin': 1 cosine waves, magnitudes linear scaled from start to end
%            'cos3lin': 3 cosine waves, magnitudes linear scaled from start to end
%            'cos5lin': 5 cosine waves, magnitudes linear scaled from start to end
%            'sin2pi':  1 sine wave, interval [0, ..., 2*pi]
% p_wn ... wavelet length (number of samples), integer, p_wn >= 50
% r_xx ... return: array of wavelet abszissas, integer array
% r_yy ... return: array of wavelet magnitudes, double array
%
function [r_xx, r_yy] = us_wavelet_generate(p_wt, p_wn)
  
  % check arguments
  if (nargin < 2)
    help us_wavelet_generate;
    error('Missing arguments!');
  end
  if (p_wn < 50)
    warning('Wavelet length < 50!\n  using wavelet length = 50 samples');
    p_wn = 50;
  end
  p_wn = fix(p_wn);
  
  % switch wavelet type
  switch (p_wt)
    case {'mh', 'mexihat', 'mexicanhat'}
      [r_yy, r_xx] = mexihat(-5, 5, p_wn);
    case {'cos1lin'}
      d2 = pi / 2;
      r_xx = linspace(-d2, d2, p_wn);
      r_yy = cos(r_xx) .* linspace(0, 1, p_wn);
    case {'cos3lin'}
      d2 = 5 * pi / 2;
      r_xx = linspace(-d2, d2, p_wn);
      r_yy = cos(r_xx) .* linspace(0, 1, p_wn);
    case {'cos5lin'}
      d2 = 9 * pi / 2;
      r_xx = linspace(-d2, d2, p_wn);
      r_yy = cos(r_xx) .* linspace(0, 1, p_wn);
    case {'sinpi'}
      r_xx = linspace(0, pi, p_wn);
      r_yy = sin(r_xx);
    case {'cospi'}
      r_xx = linspace(0, pi, p_wn);
      r_yy = cos(r_xx);
    case {'sin2pi'}
      r_xx = linspace(0, 2 * pi, p_wn);
      r_yy = sin(r_xx);
    case {'cos2pi'}
      r_xx = linspace(0, 2 * pi, p_wn);
      r_yy = cos(r_xx);
    case {'triang'}
      r_xx = linspace(0, 1, p_wn);
      r_yy = transpose(triang(p_wn));
    otherwise
      error('Unknown wavelet function type!\n  type = %s', p_wt);
  end
  
end
