% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Usage: [r_cc, r_wx, r_mv, r_mi, r_mj] = us_wavelet_xcorr(p_yy, p_wt, p_wr)
%
% p_yy ... array of signal magnitudes, double array
% p_wt ... wavelet type, string
% p_wr ... wavelet size range (number of samples), integer array [n_min, ..., n_max]
% r_cc ... return: cross correlation matrix (normalized)
% r_wx ... return: array of wavlet maxima indeces
% r_mv ... return: maximum value of correlation, double
% r_mi ... return: row index of maximum, integer
% r_mj ... return: column index of maximum, integer
%
function [r_cc, r_wx, r_mv, r_mi, r_mj] = us_wavelet_xcorr(p_yy, p_wt, p_wr)
  
  % check arguments
  if (nargin < 3)
    help us_wavelet_xcorr;
    error('Missing function arguments!');
  end
  if (isempty(p_yy))
    help us_wavelet_xcorr;
    error('Magnitude series is empty!');
  end
  if (isempty(p_wt))
    help us_wavelet_xcorr;
    error('Wavelet type is empty!');
  end
  if (isempty(p_wr))
    help us_wavelet_xcorr;
    error('Wavelet size range array is empty!');
  end
  
  % number of samples (signal)
  n_yy = max(size(p_yy));
  
  % extend signal with zeros
  yy = [transpose(p_yy(:)), zeros(1, p_wr(end))];
  
  % run wavelet analysis
  r_cc = zeros(max(size(p_wr)), n_yy);
  cnt = 1;
  for wn = p_wr
    % compute wavelet
    [wxx, wyy] = us_wavelet_generate(p_wt, wn);
    wn2 = fix(wn / 2);
    % compute cross correlation with wavelet
    for i = 1 : n_yy
      r_cc(cnt, i + wn2) = sum(yy(i : (i + wn - 1)) .* wyy);
    end
    cnt = cnt + 1;
  end
  
  % find maximum and indeces of maximum
  r_mv = max(max(r_cc)); % max value
  [~, r_mi] = max(max(r_cc, [], 2)); % row index
  [~, r_mj] = max(max(r_cc, [], 1)); % column index
  
  % normalize matrix
  r_cc = r_cc / r_mv;
  
  % compute maxima array
  [~, r_wx] = max(r_cc, [], 2);
  r_wx = transpose(r_wx);
  
end
