% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Analyse all grid point signals with calibrated speed of sound
%
% Usage: [r_ds] = us_analyse(p_ds, p_ts, p_zs, p_cf, p_wl, p_ss)
%
% p_ds ... grid data, structure
% p_ts ... truncate right-hand-side of signal (number of samples), integer
% p_zs ... set initial signal magnitudes to zero (number of samples), integer
% p_cf ... cut-off frequency of lowpass filter (unit: Hz), integer
% p_ss ... calibrated speed of sound (unit: m/s), double
% r_ds ... return: updated grid data, structure
%
% Note: see us_createstruct for details on structure fields
%
function [r_ds] = us_analyse(p_ds, p_ts, p_zs, p_cf, p_wl, p_ss)
  
  % check arguments
  if (nargin < 6)
    help us_analyse;
    error('Missing function arguments!');
  end
  
  % check structure type
  if (not(us_checkstruct(p_ds, 2)))
    error('Given data structure is not of type grid data structure!');
  end
  
  % wavelet type and length range
  wt = 'cos3lin';
  wr = [p_wl - 50, p_wl + 50];
  
  % init return value
  r_ds = p_ds;
  
  % initial message
  disp('Step 2: Analyse all grid point signals with calibrated speed of sound');
  
  % run analysis for all grid points
  arr_dist = zeros(1, p_ds.nx * p_ds.ny);
  cnt = 1;
  for i = 1 : p_ds.nx
    for j = 1 : p_ds.ny
      fprintf('... analysing grid point signal, row = %d, column = %d\n', i, j);
      ds = p_ds.arr_ds(i, j);
      ds = us_update_param(ds, [], p_ss);
      ds = us_cond_truncright(ds, p_ts);
      ds = us_cond_zeroleft(ds, p_zs);
      ds = us_cond_fltbutter(ds, p_cf);
      as = us_wavelet_analyse(ds, wt, wr);
      as.ss = ds.ss;
      r_ds.arr_ds(i, j) = ds;
      us_plot(as, 3);
      dsmp = as.mj - fix(as.wl(as.mi) * 0.5);
      dlen = fix(as.ss * (dsmp / as.sr) * 1e3);
      arr_dist(cnt) = dlen;
      fprintf('... GP(%d,%d): d = %d [mm]\n', i, j, dlen);
      cnt = cnt + 1;
    end
  end
  
  % print average of distances
  disp(' ');
  fprintf('Average distance of %d grid points: %d [mm]\n', p_ds.nx * p_ds.ny, fix(mean(arr_dist)));
  
  % plot wavelet function
  us_plot_wavelet(wt, p_wl);
  
end
