% ultrasonic - analyse and plot data from ultrasonic reflection measurements
% Copyright 2023 Jakob Harden (jakob.harden@tugraz.at, Graz University of Technology, Graz, Austria)
% License: MIT
% This program was written in the course of the lecture 'Anwendung ZfP- und Monitoringverfahren [206.554]'
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
% documentation files (the “Software”), to deal in the Software without restriction, including without 
% limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
% the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
% 
% THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
% TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
%
% Calibrate speed of sound of medium to known measurement distance
%
% Usage: [r_ss] = us_calibrate(p_ds, p_ix, p_iy, p_ts, p_zs, p_cf, p_wl, p_di, p_ns)
%
% p_ds ... grid data, structure
% p_ix ... column index of grid point to use for calibration, integer
% p_iy ... row index of grid point to use for calibration, integer
% p_ts ... truncate right-hand-side of signal (number of samples), integer
% p_zs ... set initial signal magnitudes to zero (number of samples), integer
% p_cf ... cut-off frequency of lowpass filter (unit: Hz), integer
% p_wl ... estimated wavelet length (number of samples), integer
% p_di ... known distance (unit: m), double
% p_ns ... start index of best matching wavelet (number of samples), integer
% r_ss ... return: speed of sound (unit: m/s), double
%
% Note: see us_createstruct for details on structure fields
%
function [r_ss] = us_calibrate(p_ds, p_ix, p_iy, p_ts, p_zs, p_cf, p_wl, p_di, p_ns)
  
  % check arguments
  if (nargin < 1)
    help us_calibrate;
    error('Missing grid data structure!');
  end
  if (nargin < 3)
    p_ix = 1;
    p_iy = 1;
    stage = 1;
  else
    stage = nargin - 2;
  end
  
  % check structure type
  if (not(us_checkstruct(p_ds, 2)))
    error('Given data structure is not of type grid data structure!');
  end
  
  % wavelet type
  wt = 'cos3lin';
  
  % init return values
  r_ss = 0.0;
  
  % initial message
  fprintf('Step 1: Calibrate speed of sound to known measurement distance\n');
  
  if (stage > 0)
    % print stage info
    fprintf('Step 1.1: select grid point\n');
    fprintf('          row = %d, column = %d\n', p_iy, p_ix);
    % select grid point
    ds1 = p_ds.arr_ds(p_iy, p_ix);
    fprintf('          next step: truncate right-hand-side of signal, 4000 samples is a good choice\n');
    fprintf('          command: vs = us_calibrate(ds, %d, %d, 4000)\n', p_iy, p_ix);
  end
  
  if (stage > 1)
    % print stage info
    fprintf('Step 1.2: truncate right-hand-side of signal\n');
    fprintf('          remaining samples: nr = %d [samples]\n', p_ts);
    % truncate right-hand-side of signal
    ds2 = us_cond_truncright(ds1, p_ts);
    fprintf('          next step: set initial signal magnitudes to zero, 1700 samples is a good choice\n');
    fprintf('          command: vs = us_calibrate(ds, %d, %d, %d, 1700)\n', p_iy, p_ix, p_ts);
  end
  
  if (stage > 2)
    % print stage info
    fprintf('Step 1.3: set initial signal magnitudes to zero\n');
    fprintf('          initial samples: nz = %d [samples]\n', p_zs);
    % set initial signal magnitudes to zero
    ds3 = us_cond_zeroleft(ds2, p_zs);
    fprintf('          next step: filter signal magnitudes, 0.1 * sample rate (2.5e6) is a good choice\n');
    fprintf('          command: vs = us_calibrate(ds, %d, %d, %d, %d, 2.5e6)\n', p_iy, p_ix, p_ts, p_zs);
  end
  
  if (stage > 3)
    % print stage info
    fprintf('Step 1.4: filter signal magnitudes\n');
    fprintf('          filter: Butterworth, lowpass, 1st order\n');
    fprintf('          cutoff frequency: fc = %d [Hz]\n', p_cf);
    % pre-conditioning, lowpass filter
    ds4 = us_cond_fltbutter(ds3, p_cf);
    fprintf('          next step: wavelet analysis, guess length of reflection (3 waves), 600 is a good choice\n');
    fprintf('          command: vs = us_calibrate(ds, %d, %d, %d, %d, %d, 600)\n', p_iy, p_ix, p_ts, p_zs, p_cf);
  end
  
  if (stage > 4)
    % define wavelet length range
    wl0 = p_wl - 300;
    wl1 = p_wl + 300;
    wlr = [wl0, wl1];
    % print stage info
    fprintf('Step 1.5: wavelet analysis\n');
    fprintf('          wavelet type: %s\n', wt);
    fprintf('          wavelet size range: sr = (%d,...,%d) [samples]\n', wl0, wl1);
    % wavelet analysis
    ds5 = us_wavelet_analyse(ds4, wt, wlr);
    fprintf('          next step: calibrate speed of sound to known distance, d = 0.25 and number of samples from max in graph\n');
    fprintf('          command: vs = us_calibrate(ds, %d, %d, %d, %d, %d, %d, 0.25, 2007)\n', p_iy, p_ix, p_ts, p_zs, p_cf, p_wl);
  end
  
  if (stage > 5)
    % print stage info
    fprintf('Step 1.6: calibrate speed of soundto known distance\n');
    fprintf('          known distance: d = %.3f [m]\n', p_di);
    fprintf('          best matching wavelet start point: ns = %d [samples]\n', p_ns);
    % compute speed of sound
    r_ss = us_calc_speed(ds4, p_di, p_ns);
    % print speed of sound on screen
    fprintf('Speed of sound: v_s = %.1f [m/s]\n', r_ss);
    disp(' ');
    fprintf('next step: analyse all grid point signals with calibrated speed of sound\n');
    fprintf('command: us_analyse(ds, %d, %d, %d, %d, %.1f)\n', p_ts, p_zs, p_cf, p_wl, r_ss);
  end
  
  % plot results
  switch (stage)
    case 1
      us_plot(ds1, 1);
    case 2
      us_plot(ds2, 1);
    case 3
      us_plot(ds3, 1);
    case 4
      us_plot(ds4, 1);
    case 5
      us_plot(ds5, 1);
  end
  
end
